﻿using System;
using System.Linq;
using System.Reactive.Linq;
using System.Threading;
using System.Windows;
using System.Windows.Controls;
using SilverlightApplication1.Observables;
using SilverlightApplication1.Views;

namespace SilverlightApplication1
{
    public partial class Sensor : Page
    {
        public Sensor()
        {
            InitializeComponent();
        }

        ObservableSensor sensor = new ObservableSensor();

        private void Start_Click(object sender, RoutedEventArgs e)
        {
            ThreadPool.QueueUserWorkItem(_ => sensor.StartSensor());
        }

        private void Stop_Click(object sender, RoutedEventArgs e)
        {
            sensor.StopSensor();
        }

        IDisposable activeLowValueSensor;

        private void FilterLowValue_Click(object sender, RoutedEventArgs e)
        {
            if (activeLowValueSensor == null)
            {
                var lowValueSensors = from s in sensor
                                      where s.SensorValue < 3
                                      select s.SensorValue;
                activeLowValueSensor = lowValueSensors.Subscribe(new ConsoleObserver<double>());
            }
            else
            {
                activeLowValueSensor.Dispose();
                activeLowValueSensor = null;
            }
        }

        private void FilterFirstType_Click(object sender, RoutedEventArgs e)
        {
            var listSource = new ObserverCollection<double>();

            var typeSensors = from s in sensor
                              where s.SensorType == "2"
                              select s.SensorValue;
          
            FilteredList.ItemsSource = listSource;

            typeSensors.ObserveOn(SynchronizationContext.Current).Subscribe(listSource);
  
        }

        private void QueryAny_Click(object sender, RoutedEventArgs e)
        {
            var anySensor = sensor.Any(v => v.SensorValue > 17);
            anySensor.ObserveOn(SynchronizationContext.Current).Subscribe(v => MessageBox.Show(v.ToString()));
        }

        private void Heartbeat_Click(object sender, RoutedEventArgs e)
        {
            var heartbeat = sensor.Buffer(TimeSpan.FromSeconds(3));

            heartbeat.ObserveOn(SynchronizationContext.Current)
                .Subscribe(v => Heartbeat.Content = v
                                    .Where(val => val.SensorType == "2")
                                    .Count());

        }
    }
}
