﻿using System;
using System.Collections.ObjectModel;
using System.Reactive.Linq;
using System.Threading;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Navigation;

namespace SilverlightApplication1.Views
{
    public partial class WcfTester : Page
    {
        public WcfTester()
        {
            InitializeComponent();
        }

        // Executes when the user navigates to this page.
        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            ReactiveService.ILongRunningService isvc = new ReactiveService.LongRunningServiceClient();
            var origContext = SynchronizationContext.Current;
            
            IObservable<string> inputChanges =
               (from keyup in Observable.FromEventPattern<KeyEventArgs>(this.Input, "KeyUp")
                select Input.Text)
               .Throttle(TimeSpan.FromMilliseconds(200))
               .ObserveOn(SynchronizationContext.Current)
               ;

            var sortObservable = Observable.FromAsyncPattern<string, string>(isvc.BeginSortIt, isvc.EndSortIt);

            var results = (from change in inputChanges
                          from res in sortObservable.Invoke(change)
                            .ObserveOn(origContext)
                            .TakeUntil(inputChanges)
                          select res)
                          .DistinctUntilChanged()
                          ;

            ObservableCollection<string> resultList = new ObservableCollection<string>();
            destList.ItemsSource = resultList;

            results.Subscribe(
                res => resultList.Add(res),
                err => ErrorBox.Text = err.ToString());
        }
    }
}
